#ifndef __SYMMETRY_H
#define __SYMMETRY_H 1

#include "config.h"  /* NN (empty if make test used) */
#if !defined(MaxSym)
#define MaxSym (8*NN) /* try to set minimum (less mem, faster) */
#endif
/* #include "hilbert.h" */ /* t_sites */
/* #include "spins.h" */

/* test and if neccessary create up to num symmetries P */


/* --- ABEL(NOABEL) Nutzung abelscher(nichtabelscher) Symmetriegruppe --- */
#if !defined(CONFIG_NOABEL) && !defined(CONFIG_ABEL)
#define CONFIG_NOABEL 1		/* CONFIG_ABEL or CONFIG_NOABEL */
#endif

/* --- S1SYM - special handling of spin-1 ((n>1)/2) symmetry --- */
#if !defined(CONFIG_S1SYM) && !defined(CONFIG_NOS1SYM)
#define CONFIG_S1SYM  1		/* CONFIG_S1SYM or CONFIG_NOS1SYM */
#endif
/*
 *  CONFIG_S1SYM useful for spin-1 systems with a lot of symmetries
 *             - does reduce the time needed for scanning for symmetries
 *             - this option should vanish in later versions (js Okt.98)
 */

/* to store arrays of site numbers, we take smallest possible type */
#if NN<128
#if 1  /* 256*64*1B=16kB fits to L1 but no advantage seen */
typedef signed char t_sites; /* less size, J-PC loopN=v8=19.7ns*1.6GHz */
#else
typedef int t_sites; /* better speed (if int array fits L1) 18.9ns*1.6GHz */
#endif
#elif NN<32768
typedef signed short t_sites;
#else
typedef int t_sites; /* speed? */
#endif
#if VecType & 4				/* new 1998-166 JS */
/* typedef Complex     tphase; */       /* exp(I*2*Pi*k/l) */
/* ToDo: better store exp-argument phase as integer in units of l (KGV l's) */
typedef cplx     tphase;		/* exp(I*2*Pi*k/l) */
#else
#if 1   /* size (factor 4) versus speed (2%?) */
typedef signed char tphase;		/* only -1 or +1 possible */
#else
typedef signed int  tphase;		/* only -1 or +1 possible */
#endif
#endif
/*  --- int=4*char usually signed/unsigned? pc-specifisch! 
    --- evl. tsym.w als array[k][l] zum speicher sparen
    --- large struct!!! do not use as local variable! Nov99
 */
#define MaxNumSL 8  /* maximum number of sublattice configurations (j1,..) */
struct tsym {			/* Symmetrien	*/
  t_sites i[MaxSym][NN] 	/*  Permutation j->i[j]	*/
/* 2015-06-09 gcc -O3 -fopt-info -fopt-info-vec-missed, says unknown align */
 __attribute__ ((aligned (64)));
 /* i should fit and stay in L1 256*64*1B=16kB */
  t_sites k[MaxSym];            /*  k => exp(i*2Pi*k)	*/
  tphase  w[MaxSym]             /*  weight exp(i*2Pi*k)=1,-1 l=2 or 0 */
/* 2015-06-09 gcc -O3 -fopt-info -fopt-info-vec-missed, says unknown align */
 __attribute__ ((aligned (64))); /* does not help! */
  t_sites l[MaxSym];            /*  cycle length k = 0,1,2,..,l-1 */
  char    g[MaxSym];            /*  1 = generators of sym-group */
  int	numsym,numall,          /*  Sym, 1<<Sym ???     */
        wud, wud_saved,         /*  wud=+1,0=no,-1 ToDo: kud= -1=no,0,1 */
        init;                   /* ToDo: description ??? */
  int num_lm;			/*  number of LM-spin-sublattices 1..nn */
                                /* sym.numlm!=nn for noLMs or s=1/2 only */
  int type_lm; /* 2016 sym_tU.txt  0 = JJ.maxS(S+1), 1 = tU.maxJ(J+1) */
  /* ToDo: may be some models exist with mixed lm-type per li: lk? */ 
  int num_g;                    /* number of sym-generators  2016-01 */
  t_sites lm[NN];		/* lm-group idx */
  t_sites li[NN];		/* index within lm-group */
  t_sites ls[NN];		/* size of lm-group */
  t_sites ln[NN];		/* point to next lm-member v2.21 */
  t_sites lp[NN];		/* point to prev lm-member v2.44 */
  /* ToDo: lk[NN];     2016-01 tU equivalent of dimer-plaquette-chain?
   do we have ground states where some li's dont have not maxS,maxJ or mix */
  int     corr[NN][NN];		/* array of sym. pair correlations */
  t_sites dcorr[NN*NN][2];	/* different pair correlations */
  int   num_dcorr;		/* number of -//- (0..NN*(NN-1)) */
  /* used for the Marshall rule and SL-magnetization */
  int   subl;			/* sublattice number 0..MaxNumSL-1 */
  int   sl[MaxNumSL][NN];	/* sublattice index ABAB AABB ... */
  
  int   inverse[MaxSym];		/* point to P^(-1) */
  int   kgen[MaxSym];			/* Vorgaben k fuer Generatoren */
};

extern struct tsym sym
/* 2015-06-09 gcc -O3 -fopt-info -fopt-info-vec-missed, says unknown align */
;// __attribute__ ((aligned (64))); /* does not help */

int symcreate(int mpi_i);
int test_all_sym();

#endif
